@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v1.0.0
+## Version v1.0.1
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
-## Modified 2021/05/14
+## Modified 2021/05/18
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -23,7 +23,7 @@ BEGIN
     use Scalar::Util ();
     use List::Util ();
     use Want ();
-    our $VERSION = 'v1.0.0';
+    our $VERSION = 'v1.0.1';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -46,7 +46,9 @@ sub import
     {
         $hash->{is_tied} = 1;
     }
-    *{"${class}::caller"} = \&{"Nice::Try::caller"};
+    # 2021-05-17 (Jacques): the following was a bad idea as it was indiscriminate and 
+    # would also affect use of caller outside of try-catch blocks
+    # *{"${class}::caller"} = \&{"Nice::Try::caller"};
     filter_add( bless( $hash => ( ref( $this ) || $this ) ) );
 }
 
@@ -55,13 +57,27 @@ sub unimport
     filter_del();
 }
 
-sub caller(;$)
+sub caller($;$)
 {
+    my $where = shift( @_ );
     my $n = shift( @_ );
-    my @info = defined( $n ) ? CORE::caller( int( $n ) + 2 ) : CORE::caller(3);
+    # Offsetting our internal call frames
+    my $map = 
+    {
+    try => 3,
+    catch => 2,
+    finally => 5,
+    };
+    my @info = defined( $n ) ? CORE::caller( int( $n ) + $map->{ $where } ) : CORE::caller( 1 + $map->{ $where } );
     return( @info );
 }
 
+sub caller_try { return( &Nice::Try::caller( try => @_ ) ); }
+
+sub caller_catch { return( &Nice::Try::caller( catch => @_ ) ); }
+
+sub caller_finally { return( &Nice::Try::caller( finally => @_ ) ); }
+
 sub filter
 {
     my( $self ) = @_ ;
@@ -548,6 +564,7 @@ sub _parse
         if( scalar( @$fin_block_ref ) )
         {
             my $fin_def = $fin_block_ref->[0];
+            $self->_process_caller( finally => $fin_def->{block} );
             ## my $finally_block = $fin_def->{block}->content;
             my $finally_block = $self->_serialize( $fin_def->{block} );
             $finally_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
@@ -596,6 +613,7 @@ EOT
             }
             
             $self->_process_loop_breaks( $try_def->{block} );
+            $self->_process_caller( try => $try_def->{block} );
             
             ## my $try_block = $try_def->{block}->content;
             my $try_block = $self->_serialize( $try_def->{block} );
@@ -828,6 +846,7 @@ EOT
                 ## $self->_message( 3, "\$i = $i, \$total_catch = $total_catch and cond = '$cond'" );
                 ## my $block = $cdef->{block}->content;
                 $self->_process_loop_breaks( $cdef->{block} );
+                $self->_process_caller( catch => $cdef->{block} );
                 my $block = $self->_serialize( $cdef->{block} );
                 $block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
                 my $catch_section = '';
@@ -1176,6 +1195,38 @@ EOT
     return( $elem );
 }
 
+# .Element: [11] class PPI::Token::Word, value caller
+# .Element: [11] class PPI::Structure::List, value (1)
+#
+# ..Element: [12] class PPI::Token::Word, value caller
+# ..Element: [12] class PPI::Token::Structure, value ;
+
+sub _process_caller
+{
+    my $self = shift( @_ );
+    my $where = shift( @_ );
+	my $elem = shift( @_ ) || return( '' );
+    return( $elem ) if( !$elem->children );
+    foreach my $e ( $elem->elements )
+    {
+        $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $e->content );
+        my $class = $e->class;
+        if( $class eq 'PPI::Token::Word' && $e->content =~ /^(?:CORE\::)?(?:GLOBAL\::)?caller$/ )
+        {
+            $self->_message( 4, "Found caller, replacing with ", 'Nice::Try::caller_' . $where );
+            $e->set_content( 'Nice::Try::caller_' . $where );
+        }
+        
+        if( $e->can('elements') && $e->elements )
+        {
+            $self->_process_caller( $where => $e );
+        }
+    }
+    # $self->_message( 5, "Element now is: '$elem'" );
+    # $self->_browse( $elem );
+    return( $elem );
+}
+
 sub _process_loop_breaks
 {
     my $self = shift( @_ );
@@ -1193,6 +1244,10 @@ sub _process_loop_breaks
             # $self->_message( 6, "Skipping it. Its first word was '", $e->first_element->content, "'" );
             next;
         }
+        elsif( $class eq 'PPI::Token::Word' && $e->content =~ /^(?:CORE\::)?(?:GLOBAL\::)?(next|last|redo)$/ )
+        {
+            $e->set_content( qq{CORE::return( '__} . uc( $1 ) . qq{__' )} );
+        }
         elsif( $class eq 'PPI::Statement::Break' )
         {
             my $words = $e->find( 'PPI::Token::Word' );
@@ -1205,7 +1260,8 @@ sub _process_loop_breaks
             if( ( scalar( @$words ) == 1 ||
                   ( scalar( @$words ) > 1 && $words->[1]->content =~ /^(for|foreach|given|if|unless|until|while)$/ )
                 ) && 
-                ( $words->[0]->content eq 'next' ||
+                (
+                  $words->[0]->content eq 'next' ||
                   $words->[0]->content eq 'last' ||
                   $words->[0]->content eq 'redo' ) )
             {
@@ -1246,7 +1302,7 @@ sub _process_loop_breaks
         }
     }
     # $self->_message( 5, "Element now is: '", sub{ $elem->content }, "'" );
-    $self->_message( 5, "Element now is: '$elem'" );
+    # $self->_message( 5, "Element now is: '$elem'" );
     # $self->_browse( $elem );
     return( $elem );
 }
@@ -1576,7 +1632,7 @@ Also since version 1.0.0, L<Nice::Try> is context aware:
 
 =head1 VERSION
 
-    v1.0.0
+    v1.0.1
 
 =head1 DESCRIPTION
 
@@ -1714,7 +1770,7 @@ Would produce:
 
 =item * Supports a C<finally> block called in void context for cleanup for example
 
-=item * L<Nice::Try> is rich context aware, which means it can provide you with a super granual context on how to return data back to the caller based on the caller's expectation, by using a module like L<Want>.
+=item * L<Nice::Try> is rich context aware, which means it can provide you with a super granular context on how to return data back to the caller based on the caller's expectation, by using a module like L<Want>.
 
 =item * Call to L<perlfunc/caller> will return the correct entry in call stack
 
@@ -2112,7 +2168,7 @@ Normally, you would write something like this, and it works as always:
         }
     }
 
-THe above is nice, but how do you differentiate cases were your caller wants a simple returned value and the one where the caller wants an object for chaining purpose, or if the caller wants an hash or array reference in return?
+The above is nice, but how do you differentiate cases were your caller wants a simple returned value and the one where the caller wants an object for chaining purpose, or if the caller wants an hash or array reference in return?
 
 For example:
 
@@ -2140,12 +2196,12 @@ Now, you can do the following:
             {
                 return( $name ); # regular string
             }
-            # same as !defined( wantarray() )
+            # same as if( !defined( wantarray() ) )
             elsif( want('VOID') )
             {
                 return;
             }
-            # For the other context below, wantarray is of no help
+            # For the other contexts below, wantarray is of no help
             if( want('OBJECT') )
             {
                 return( $obj ); # useful for chaining
@@ -2198,7 +2254,7 @@ Other cases are:
 
 See L<Want> for more information on how you can benefit from it.
 
-Currently lvalues are no implemented and will be in future releases.
+Currently lvalues are no implemented and will be in future releases. Also note that L<Want> does not work with tied object and overloaded classes. L<Nice::Try> detects this and disable automatically support for L<Want> if the object is overloaded.
 
 Also, for this rich context awareness to be used, obviously try-catch would need to be inside a subroutine, otherwise there is no rich context other than the one the regular L<perlfunc/wantarray> provides.
 
